import java.io.*;
import java.util.*;

public class MakeCityGrammar {

        private static SortedMap<String,String> readStopNames(String file) throws IOException {
                LineNumberReader r = new LineNumberReader(new FileReader(file));

                TreeMap<String,String> stops = new TreeMap<String,String>();
                String line;
                while ((line = r.readLine()) != null) {
                        if (line.matches("^\\s*$"))
                                continue;
                        String[] ss = line.split("\\s*:\\s*");
                        if (ss.length != 2)
                                throw new IOException("Syntax error in " + file 
                                                      + ", line " + r.getLineNumber() + ": " 
                                                      + line);
                        stops.put(ss[0], ss[1]);
                }

                r.close();

                return stops;
        }


        private static String toEng(String name) {
                // FIXME: figure out right pronunciation

                // FIXME: this could be more efficiently done in place
                // but who cares?
                name = name.toLowerCase();

                name = name.replace('','a');
                name = name.replace('','a');
                name = name.replace('','o');
                name = name.replace('','e');

                return name;
        }

        private static String toSwe(String name) {
                return name.toLowerCase();
        }

        private static String toLang(String lang, String fun, String name) {
                if (lang.equals("Eng")) {
                        return toEng(name);
                } else if (lang.equals("Swe")) {
                        return toSwe(name);
                } else if (lang.equals("Labels")) {
                        return fun;
                } else if (lang.equals("Names")) {
                        return name;
                } else
                        throw new RuntimeException("Unknown language: " + lang);
        }

        private static PrintWriter openFile(String name) throws IOException {
                System.err.println("Writing " + name);
                return new PrintWriter(new FileWriter(name));
        }

        public static void main(String[] args) throws IOException {
                if (args.length != 2) {
                        System.err.println("Usage: java MakeCityGrammar <grammar name> <stop name file>");
                        System.exit(2);
                }

                String absName = args[0];
                String stopsFile = args[1];
                List<String> langs = Arrays.asList("Swe", "Eng", "Labels", "Names");

                SortedMap<String,String> stops = readStopNames(stopsFile);

                PrintWriter abs = openFile(absName + ".gf");

                abs.println("abstract " + absName + " = Transport ** {");

                for (Map.Entry<String,String> stop : stops.entrySet()) {
                        // FIXME: make a better fun name
                        abs.println("fun " + stop.getKey() + " : Stop ;");
                }

                abs.println("}");
                abs.close();

                for (String lang : langs) {
                        String cncName = absName + lang;
                        PrintWriter cnc = openFile(cncName + ".gf");

                        cnc.println("concrete " + cncName + " of " + absName
                                    + " = Transport" + lang + " ** {");

                        for (Map.Entry<String,String> stop : stops.entrySet()) {
                                String fun = stop.getKey();
                                String name = stop.getValue();
                                cnc.println("lin " + stop.getKey() + " = { s = [\"" 
                                            + toLang(lang, fun, name) + "\"] } ;");
                        }

                        cnc.println("}");
                        cnc.close();
                }
        }

}
